package dev.svitan.antifed

import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.outlined.ArrowBack
import androidx.compose.material3.CircularProgressIndicator
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TextField
import androidx.compose.material3.TopAppBar
import androidx.compose.material3.TopAppBarDefaults
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.input.PasswordVisualTransformation
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import dev.svitan.antifed.ui.theme.AntiFedTheme
import io.ktor.client.call.body
import io.ktor.client.request.get
import io.ktor.http.HttpStatusCode
import kotlinx.coroutines.delay

@OptIn(ExperimentalMaterial3Api::class)
class AuthActivity : ComponentActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        enableEdgeToEdge()
        setContent {
            AntiFedTheme {
                var serverUrl by remember { mutableStateOf("") }
                var token by remember { mutableStateOf("") }

                var needToCheckUrl by remember { mutableStateOf(false) }
                var checkingUrl by remember { mutableStateOf(false) }
                var isServerUrlOk by remember { mutableStateOf(false) }

                var needToCheckToken by remember { mutableStateOf(false) }
                var checkingToken by remember { mutableStateOf(false) }
                var isTokenOk by remember { mutableStateOf(false) }

                var auths by remember { mutableStateOf(listOf<AuthDTO>()) }

                val prefs = LocalContext.current.getSharedPreferences(
                    stringResource(R.string.settings_prefs_key),
                    MODE_PRIVATE
                )
                val loadedServerUrl = prefs.getString(getString(R.string.server_url_key), "") ?: ""
                if (loadedServerUrl.isNotBlank()) {
                    serverUrl = loadedServerUrl
                }
                isServerUrlOk = prefs.getBoolean(getString(R.string.server_url_okay_key), false)

                fun serverUrlMatches(): Boolean {
                    var re = Regex(
                        """https?://[a-zA-Z0-9\\.]+\\.[a-zA-Z0-9]+""",
                        RegexOption.DOT_MATCHES_ALL
                    )
                    return re.containsMatchIn(serverUrl)
                }

                LaunchedEffect(serverUrl) {
                    if (!needToCheckUrl) return@LaunchedEffect
                    else {
                        isServerUrlOk = false
                        with(prefs.edit()) {
                            putBoolean(getString(R.string.server_url_okay_key), false)
                            apply()
                        }
                    }

                    if (serverUrl.isBlank()) return@LaunchedEffect
                    if (!serverUrlMatches()) return@LaunchedEffect

                    delay(1000)
                    checkingUrl = true
                    val response = client.get(serverUrl)

                    checkingUrl = false
                    needToCheckUrl = false
                    isServerUrlOk = response.status == HttpStatusCode.OK
                    with(prefs.edit()) {
                        putBoolean(getString(R.string.server_url_okay_key), isServerUrlOk)
                        apply()
                    }
                }

                LaunchedEffect(token) {
                    if (!needToCheckToken) return@LaunchedEffect
                    else isTokenOk = false

                    if (!isServerUrlOk) return@LaunchedEffect
                    if (token.isBlank()) return@LaunchedEffect

                    delay(1000)
                    checkingToken = true

                    val response = client.get("$serverUrl/auth")
                    if (response.status != HttpStatusCode.OK) {
                        checkingToken = false
                        needToCheckToken = false
                        isTokenOk = false
                        return@LaunchedEffect
                    }

                    auths = response.body<List<AuthDTO>>()
                    checkingToken = false
                    needToCheckToken = false
                    isTokenOk = true
                }

                Scaffold(
                    topBar = {
                        TopAppBar(
                            colors = TopAppBarDefaults.topAppBarColors(
                                titleContentColor = MaterialTheme.colorScheme.primary
                            ),
                            title = {
                                Text(stringResource(R.string.auth))
                            },
                            navigationIcon = {
                                IconButton(onClick = { finish() }) {
                                    Icon(
                                        imageVector = Icons.AutoMirrored.Outlined.ArrowBack,
                                        contentDescription = stringResource(R.string.go_back)
                                    )
                                }
                            }
                        )
                    },
                    modifier = Modifier.fillMaxSize()
                ) { innerPadding ->
                    Column(
                        horizontalAlignment = Alignment.CenterHorizontally,
                        verticalArrangement = Arrangement.Center,
                        modifier = Modifier
                            .padding(innerPadding)
                            .fillMaxSize()
                    ) {
                        Text(stringResource(R.string.auth), fontSize = 24.sp)
                        Spacer(modifier = Modifier.height(16.dp))

                        TextField(
                            value = serverUrl,
                            onValueChange = {
                                needToCheckUrl = true
                                serverUrl = it
                                with(prefs.edit()) {
                                    putString(getString(R.string.server_url_key), it)
                                    apply()
                                }
                            },
                            label = { Text(stringResource(R.string.server_url)) },
                            singleLine = true,
                            isError = !isServerUrlOk,
                            trailingIcon = {
                                if (checkingUrl)
                                    CircularProgressIndicator(
                                        modifier = Modifier.width(36.dp),
                                        color = MaterialTheme.colorScheme.secondary,
                                        trackColor = MaterialTheme.colorScheme.surfaceVariant
                                    )
                            }
                        )
                        Spacer(modifier = Modifier.height(12.dp))

                        TextField(
                            value = token,
                            onValueChange = {
                                needToCheckToken = true
                                token = it
                            },
                            label = { Text(stringResource(R.string.token)) },
                            singleLine = true,
                            isError = token.isBlank(),
                            visualTransformation = PasswordVisualTransformation(),
                            keyboardOptions = KeyboardOptions(keyboardType = KeyboardType.Password)
                        )
                        Spacer(modifier = Modifier.height(12.dp))

                        Text("auths: $auths")
                    }
                }
            }
        }
    }
}
